﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Syncfusion.XlsIO;
using System.Drawing;
using System.Data;
using Syncfusion.XlsIO.Implementation.Charts;

namespace SampleXlsIO.Controllers
{
    public partial class XlsIOController : Controller
    {
        #region Getting Started
        public ActionResult XlsIOFeatures(string SaveOption)
        {
            if (SaveOption == null)
                return View();
            //Step 1 : Instantiate the spreadsheet creation engine.
            ExcelEngine excelEngine = new ExcelEngine();
            //Step 2 : Instantiate the excel application object.
            IApplication application = excelEngine.Excel;
            
            // Creating a worksheet as 0 index with Name Q1
            IWorkbook workbook = excelEngine.Excel.Workbooks.Create(new string[] {"Q1"});
            //Access first worksheet from the workbook instance.
            IWorksheet worksheet = workbook.Worksheets[0];
            worksheet.Range["A1"].Text = "Attitudes & Perceptions";
            worksheet.Range["A1"].CellStyle.Font.Bold = true;
            worksheet.Range["A1"].CellStyle.Font.Size = 18;
            int numRows = 3;

            IList<BusinessCustomer> customers = new List<BusinessCustomer>();
            customers.Add(new BusinessCustomer("I am kept well-informed about changes in the organization that affect my work group.", 0, 0, 0, 4, 2, 4.33));
            customers.Add(new BusinessCustomer("I feel I am able to voice my opinions/concerns openly with my manager.", 1, 0, 1, 1, 3, 3.83));
            customers.Add(new BusinessCustomer("Issues and topics get discussed and resolved to my satisfaction.", 0, 1, 0, 4, 1, 3.83));
            customers.Add(new BusinessCustomer("I feel I am able to voice my opinions/concerns openly with my manager.", 1, 0, 1, 2, 2, 3.67));
            customers.Add(new BusinessCustomer("Communication between departments is well-coordinated and effective.", 1, 0, 1, 0, 4, 4.0));
            customers.Add(new BusinessCustomer("We are encouraged/supported to use social media (LinkedIn, Facebook, etc.) as a business-building tool.", 3, 0, 1, 1, 1, 2.50));

           
            worksheet.ImportData(customers, 3, 1, true);
            
            IList<object> questions = new List<object>();
            IList<object> avgAnswer = new List<object>();

            foreach(BusinessCustomer customer in customers.Reverse())
            {
                questions.Add(customer.Question);
                avgAnswer.Add(customer.AverageAnswer);
            }


            IChartShape chart = worksheet.Charts.Add();
            chart.ChartTitle = "Average Answer";
            IChartSerie serie = chart.Series.Add(ExcelChartType.Bar_Clustered);
            serie.EnteredDirectlyValues = avgAnswer.ToArray(); // X axis values            
            serie.EnteredDirectlyCategoryLabels = questions.ToArray(); // Y axis Values
            chart.TopRow = worksheet.UsedRange.LastRow + 1;
            chart.BottomRow += worksheet.UsedRange.LastRow + 1;
            chart.PrimaryValueAxis.Title = "Average Answer";
            serie.Name = "Average Answer";
            chart.PrimaryValueAxis.Font.Bold = false;
            chart.Legend.Delete();
            chart.ChartArea.Fill.BackColor = Color.White;

            //Fill series color based on values.
            FillColor(chart);

            IList<BusinessCustomer> customers1 = new List<BusinessCustomer>();
            customers1.Add(new BusinessCustomer("I consider Big Tex Bank to be diverse, reflecting different ages, ethnicities, and backgrounds", 0, 0, 2, 4, 6, 4.5));
            customers1.Add(new BusinessCustomer("Big Tex Bank encourages and supports a healthy lifestyle, and work/life balance.", 0, 0, 1, 2, 3, 4.0));
            customers1.Add(new BusinessCustomer("I have good friends", 0, 0, 1, 3, 2, 4.33));


            worksheet.ImportData(customers1, chart.BottomRow + 3 , 1 , true);

            IList<object> questions1 = new List<object>();
            IList<object> avgAnswer1 = new List<object>();

            foreach (BusinessCustomer customer in customers1.Reverse())
            {
                questions1.Add(customer.Question);
                avgAnswer1.Add(customer.AverageAnswer);
            }

            IChartShape chart1 = worksheet.Charts.Add();
            chart1.ChartTitle = "Average Answer";
            IChartSerie serie1 = chart1.Series.Add(ExcelChartType.Bar_Clustered);
            serie1.EnteredDirectlyValues = avgAnswer1.ToArray(); // X axis Values
            serie1.EnteredDirectlyCategoryLabels = questions1.ToArray(); // Y axis Values
            chart.PrimaryValueAxis.Title = "Average Answer";
            chart1.TopRow = worksheet.UsedRange.LastRow + 1;
            chart1.BottomRow += worksheet.UsedRange.LastRow + 1;
            serie1.Name = "Average Answer";
            chart1.Legend.Position = ExcelLegendPosition.Bottom;
            chart1.Legend.Delete();
            chart1.PrimaryValueAxis.Font.Bold = false;
            chart1.ChartArea.Fill.BackColor = Color.White;

            //Fill series color based on values.
            FillColor(chart1);

            workbook.SaveAs("Sample.xlsx", HttpContext.ApplicationInstance.Response, ExcelDownloadType.Open);
            workbook.Close();
            
            excelEngine.Dispose();
         
            return View();

        }
        #endregion

        public static void FillColor(IChart chart)
        {

            foreach (IChartSerie serie in chart.Series)
            {
                int dataPointCount = (serie.EnteredDirectlyValues == null) ? serie.CategoryLabels.Rows.Length : serie.EnteredDirectlyValues.Length;
                for (int dataPointPos = 0; dataPointPos < dataPointCount; dataPointPos++)
                {
                    IChartDataPoint dataPoints = serie.DataPoints[dataPointPos];
                    double value;
                    if (serie.EnteredDirectlyValues == null)
                        value = serie.CategoryLabels.Rows[dataPointPos].Number;
                    else
                        value = (double)serie.EnteredDirectlyValues.GetValue(dataPointPos);

                    if (value < 1)
                        dataPoints.DataFormat.Fill.ForeColor = Color.Red;
                    else if (value >= 1 && value < 2)
                        dataPoints.DataFormat.Fill.ForeColor = Color.Orange;
                    else if (value >= 2 && value < 3)
                        dataPoints.DataFormat.Fill.ForeColor = Color.Yellow;
                    else if (value >= 3 && value < 4)
                        dataPoints.DataFormat.Fill.ForeColor = Color.FromArgb(153, 255, 204);
                    else if (value >= 4 && value <= 5)
                        dataPoints.DataFormat.Fill.ForeColor = Color.Green;
                }
            }
            
        }
    }
    #region Helper Class
    class BusinessCustomer
    {
        #region Members
        private string m_question;
        private double m_avgAnswer;
        private int m_agree;
        private int m_stronglyAgree;
        private int m_nand;
        private int m_disagree;
        private int m_stronglyDisagree;
        private int m_total;

        #endregion

        #region Prperties
        public string Question
        {
            get
            {
                return m_question;
            }
            set
            {
                m_question = value;
            }
        }

       

        public int StronglyDisagree
        {
            get
            {
                return m_stronglyDisagree;
            }
            set
            {
                m_stronglyDisagree = value;

            }
        }
        public int Disagree
        {
            get
            {
                return m_disagree;
            }
            set
            {
                m_disagree = value;

            }
        }
        public int NAND
        {
            get
            {
                return m_nand;
            }
            set
            {
                m_nand = value;

            }
        }
        public int StronglyAgree
        {
            get
            {
                return m_stronglyAgree;
            }
            set
            {
                m_stronglyAgree = value;

            }
        }
        public int Agree
        {
            get
            {
                return m_agree;
            }
            set
            {
                m_agree = value;

            }
        }

        public int Total
        {
            get
            {
                return m_total;
            }
        }

        public double AverageAnswer
        {
            get
            {
                return m_avgAnswer;
            }
           
        }

        #endregion

        #region Intialization
        public BusinessCustomer()
        {
        }
        public BusinessCustomer(string question,int stronglyDisagree,int disagree,int nand,int stronglyAgree,int agree,double avgAnswer)
        {
            m_question = question;
            m_stronglyDisagree = stronglyDisagree;
            m_agree = agree;
            m_disagree = disagree;
            m_stronglyAgree = stronglyAgree;
            m_total = (m_stronglyAgree + m_stronglyDisagree + m_nand + m_disagree + m_agree);
            m_avgAnswer = avgAnswer;

        }
        #endregion
    }
    #endregion
}
